package Main;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import diffeq.DiffEqSolver;
import diffeq.DynamicModel;
import diode.Diode;
import diode.Diode1N4148;





// This is the main class of the project. It instantiates a model and a
// differential equation solver and then it uses the solver to step through
// time in small increments and it tabulates simulation results in a file,
// suitable for further processing.
public class CircuitSimulator {

	static final public String outFile = "C:/rossler.txt";

	public static void main(String[] args) {

		// Create instance of the Rossler-like model. This model contains the
		// equations and it knows its own order (number of independent states).
		DynamicModel rosslerLikeModel = new RosslerLikeModel();

		// Variable declarations. We have a natural frequency RosslerLikeModel.w
		// which roughly equals the omega (angular frequency) of the harmonic oscillator.
		// The time step h for integrating the differential equation is taken 1/1000 
		// of the time constant based on R and C. We write out a set of values after 
		// each 0.02 times the time constant (which means 2*pi*50 ~ 300 samples per cycle
		// around the origin). We stop simulating after 2000 time periods.
		double h = 0.001 / RosslerLikeModel.w;
		double dt = 0.02 / RosslerLikeModel.w;
		double tmax = 2000.0 / RosslerLikeModel.w;

		// Instantiate a differential equation solver and allocate space
		// for storing the state variables of the model. The number of states
		// is queried from the model itself.
		DiffEqSolver solver = new DiffEqSolver(rosslerLikeModel);
		double[] y = new double[rosslerLikeModel.getOrder()];

		BufferedWriter stream = null;
		try {
			int progress = 0;

			// Open the outpur file to which the simulation results are written
			// in tabular form. Post-processing can be done with e.g. Excel,
			// Open Office or Gnuplot.
			File f = new File(outFile);
			stream = new BufferedWriter(new FileWriter(f));

			// Set the initial condition to 0 for all state variables.
			for (int i = 0; i < y.length; i++) {
				y[i] = 0;
			}

			// Enter a loop. Only 1/4 of the simulation results is written to file.
			// The initial transient is skipped. This is especially important when
			// one wants to view limit cycles and period doublings. Only after some
			// time the output follows a trajectory close to the limit cycle and
			// for this reason the initial transient is skipped.
			for (double t = 0.0; t <= tmax; t += dt) {
				solver.solve(y, t, y, t + dt, h, 1e-8, 1e-6);
				if (t >= 0.75 * tmax) {
					for (int i = 0; i < y.length; i++) {
						stream.write("  " + y[i]);
					}
					stream.newLine();
				}

				// For each 2500 steps print a single dot. This gives feedback
				// to the user that the program is still working.
				progress++;
				if (progress % 2500 == 0) {
					System.out.print(".");
					System.out.flush();
				}
			}

			// Print message that the program is finisched.
			System.out.println(" DONE");
		} 
		catch (Exception ex) {
			System.err.println(ex);
		} 
		finally {
			if (stream != null) try {stream.close();} catch (IOException ignore) { }
		}
	}
}








// This is the class, implementing the dynamic model of the Rossler-like system
// with the opamps. All three integrators and the non-linearity are programmed
// explicitly and can easily be recognized in the source code.
// In this class only static variables are used (not really OO-paradigm), but
// as we only need one instance this is not a major issue.
class RosslerLikeModel implements DynamicModel {

	// The value of the resistor and capacitor, used in the integrators.
	static final public double R = 10000.0;
	static final public double C = 1.0e-8;
	static final public double w = 1 / (R * C);

	// The adjustable negative damping parameter, which determines
	// how fast the amplitude of the oscillator will grow if no
	// kick-back would be present.
	static final public double a = 0.215;
	
	
	// Obtain a model of the diode, used in the system.
	static final Diode diode = new Diode1N4148();
	
	

	// Set of opamp offsets. The precise values do not really matter, the effect
	// really is marginal. It is important though that one of these offsets is
	// non-zero (which is always the case in a real circuit), otherwise the
	// oscillation does not start.
	// These opamp offsets are chosen fairly arbitrarily. These numbers however
	// are values which very well can appear in cheap real opamps like the
	// TL072. Opamp offset voltages usually are in the order of magnitude of
	// a few millivolts.
	static final double xOffset = 0.005;  // Opamp offset of 5 mV
	static final double yOffset = 0.010;  // Opamp offset of 10 mV
	static final double zOffset = -0.007; // Opamp offset of -7 mV
	
	
	

	// The set of differential equations, used by the Rossler-like
	// model of opamp integrators and diode nonlinearity. Input is the
	// time and the state variables at the given time. The derivatives
	// of the state variables are computed and stored in the output
	// array xdot[].
	public void computeDerivatives(double[] xdot, double t, double[] x) {
		
		// The state vector x[] contains the three states of the Rossler-like
		// model. x[0] is the x-value, x[1] is the y-value and x[2] is the
		// z-value. Using this, one easily can recognize the equations from
		// the webpage, only having opamp offset as an additional part of
		// the simulated model.

		xdot[0] = w * (xOffset - x[1] + x[2]);
		xdot[1] = w * (yOffset + x[0] + a * x[1]);
		double diodeCurrent = diode.computeCurrent_with_resistor(x[0], 100);
		xdot[2] = w * (zOffset - x[2]) - diodeCurrent / C;
	}
	
	
	

	// The order of this set of equations is fixed and equal to 3.
	// This is the minimum order of a time-continuous system for
	// being capable of exhibiting chaotic behavior.
	public int getOrder() {
		return 3;
	}
}
