package diode;


// This model is a highly simplified diode model only taking into
// account the exponential relationship between current and applied
// voltage, and the junction resistance. Junction capacitance is not taken
// into account in this model. But even this simple model already captures
// the most essential part of the behavior of a circuit using this
// diode.

public class Diode1N4148 implements Diode {

	private double Is;
	private double A;

	
	
	// Use diode parameters, which are in the order of magnitude for
	// an 1N4148 type of signal diode at room temperature.
	public Diode1N4148() {
		Is = 1e-9;
		A = 30;
	}
	
	

	// This is a model of a diode without any series resistance.
	public double computeCurrent(double V) {
		return Is * (Math.exp(A * V) - 1.0);
	}
	
	
	

	// This is a model of a diode with a series resistor. It
	// computes the current through the series of diode and
	// resistor as function of applied voltage over the two
	// components. A very simple iterative procedure based
	// on Newton iteration is used to find the current. For
	// negative voltages, the series resistor is neglected
	// and only the reverse diode current is taken into
	// account. This function certainly is not the most
	// efficient way to compute the current, but it is very
	// simple and always works.
	public double computeCurrent_with_resistor(double V, double R) {

		if (V <= 0.0) {
			return Is * (Math.exp(A * V) - 1.0);
		}

		double Id = 0.0, newi = 0.0;
		do {
			Id = newi;
			double Vresistor = Id * R;
			double Vdiode = Math.log(1 + Id / Is) / A;
			double f = Vresistor + Vdiode - V;
			double dfdI = R + 1 / (A * (Id + Is));
			newi = Id - f / dfdI;
		} while (Math.abs(newi - Id) > 0.0000001 * Math.abs(newi));

		return Id;
	}
}


